#include "main_loop.h"

int main_loop(config_item_list *cfg_items, plugin *plugin_struct)
{
  int return_code=0;
  struct stat f_stat;
  int fd = -1;
  int data_size=0, blocks=0, rate=0, r_size=0, ok=1;
  float *data=NULL;
  char comm[FIFO_MSG_SIZE], *filename;
  SDL_Surface *screen = NULL;
  SDL_Event event;
  void (*plot)(void *, SDL_Surface *, int, float *, int);
  void *plot_cfg;
#ifdef DEBUG
  int i;
  float min, max;
  min = INFINITY;
  max = - INFINITY;
#endif

  if ((cfg_items == NULL) || (plugin_struct == NULL))
    return -1;

  plot = plugin_struct->plot;
  plot_cfg = plugin_struct->config;

  filename = (char *)((* get_config_item("fifo_file",cfg_items)).value);

  if ((stat(filename,&f_stat) == 0) && (S_ISFIFO(f_stat.st_mode)))
    {
      fprintf(stderr,"Reading data from af_analysis through fifo: '%s'\n",filename);
      if ((fd=open(filename, O_RDONLY | O_SYNC)) > -1)
	{
	  /* Initiate SDL ****************************************************/
	  if (SDL_Init(SDL_INIT_VIDEO) >= 0)
	    {	     
	      atexit(SDL_Quit);
	      if ((screen = SDL_SetVideoMode(800,600, 16, SDL_SWSURFACE)) != NULL)
		{
		  SDL_ShowCursor(SDL_DISABLE);		  
		  ok=1;
		  while (ok)
		    {		      
		      /* Process Events **************************************/
		      if (SDL_PollEvent(&event))
			{
			  switch (event.type)
			    {
			    case SDL_KEYDOWN:
			      switch (event.key.keysym.sym)				
				{
				case SDLK_f: /* Toggle Fullscreen */
				  if ( ! SDL_WM_ToggleFullScreen(screen) )
				    {
				      fprintf(stderr,"ERROR: Cannot toggle FullScreen\n");
				    }
				  break;
				case SDLK_q: /* Quit */
				  ok=0;
				  break;
				default:
				  break;
				}
			      break;
			    case SDL_QUIT:
			      ok=0;
			      break;
			    default:
			      break;
			    }
			}
		      
		      /* Read from FIFO **************************************/
		      r_size = read(fd, comm, FIFO_MSG_SIZE*sizeof(char));
		      if (r_size == -1)
			{
			  if (errno != EAGAIN)
			    {
			      /* an error occurred */
			      perror("Error reading from pipe");
			      break;
			    }
			  else
			    {
			      /* EAGAIN means that the sender didn't begin to sent data */
			      /* wait a little */
			      usleep(100000);
			    }
			}

		      /* the other side has closed */
		      if ((r_size == FIFO_MSG_SIZE*sizeof(char)) && (strcmp(comm, FIFO_CLOSED) == 0))
			{
			  break;
			}

		      /* the other side said he has began */
		      if ((r_size == FIFO_MSG_SIZE*sizeof(char)) && (strcmp(comm, FIFO_BEGIN) == 0))
			{
			  r_size = read(fd, &blocks, sizeof(int));
			  r_size += read(fd, &data_size, sizeof(int));
			  r_size += read(fd, &rate, sizeof(int));
			  if (r_size == 3*sizeof(int))
			    {			      
			      /* read the data and graph it */
			      if (data == NULL)
				{ 
				  data = (float *) calloc(data_size, sizeof(float));
				}
			      r_size = read(fd, data, sizeof(float)*data_size);

			      if (r_size == sizeof(float)*data_size)
				{
#ifdef DEBUG
				  /* get the maximum and minimum value */
				  for (i=0; i < data_size; i++) 
				    { 
				      if (min > data[i]) min = data[i];
				      if (max < data[i]) max = data[i];
				    } 
#endif					  
				  /* Plot ********************************/
				  (*plot)(plot_cfg, screen, data_size, data, rate);
				}
			      else
				{
				  fprintf(stderr,"We was supposed to read %d bytes, but %d was read\n",data_size*sizeof(float), r_size);
				}
			    }			  
			  
			  /* read until we get msg FIFO_END */
			  while ( ((r_size = read(fd, comm, FIFO_MSG_SIZE*sizeof(char))) == FIFO_MSG_SIZE*sizeof(char)) ||
				  ((r_size < 0) && (errno == EAGAIN))
				  )
			    {
			      if ((r_size == FIFO_MSG_SIZE*sizeof(char)) && (strcmp(comm,FIFO_END) == 0))
				break;
			    }
			}
		    }
			  
		}
	      else
		{
		  fprintf(stderr,"Couldn't create SDL surface %s\n",SDL_GetError());
		  return_code = -4;
		}
	    }
	  else
	    {
	      fprintf(stderr,"Couldn't initialize SDL %s\n",SDL_GetError());
	      return_code = -3;
	    }
	}
      else
	{
	  fprintf(stderr,"Couldn't open fifo: %s.\n",filename);
	  perror(NULL);
	  return_code=-2;
	}
    } 
  else
    {
      return_code=-1;
    }

  if (data != NULL) free(data);
  
#ifdef DEBUG
  printf("Min=%5g    Max=%5g\n",min,max);
#endif

  return return_code;
}
