#include "plugin.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int unload_plugin(plugin *plugin_struct)
{
  if ((plugin_struct == NULL) || (plugin_struct->handle == NULL))
    return -1;
  else
    {
      dlclose(plugin_struct->handle);
      plugin_struct->handle = NULL;
    }
  return 0;
}


int load_plugin(char *name, plugin **plugin_struct)
{
  void *handle = NULL;
  plugin *(*init_plugin)(void) = NULL;
  char *p_name = NULL;
  char *error = NULL;
  char *libdir = NULL;
  int len;

#ifdef LIBDIR
  libdir = LIBDIR;
#endif

  if ((name == NULL) || (plugin_struct == NULL))
    return -1;

  len = 0;
  if ((name[0] != '/') && (libdir != NULL))
    len = strlen(libdir);   

  /* name is terminated with .so? */
  len += strlen(name);
  if ((name[len-3] == '.') && (name[len-2] == 's') && (name[len-1] == 'o'))
    {
      /* Yes, just copy */
      p_name = (char *) calloc(len+1, sizeof(char));
      if ((libdir != NULL) && (name[0] != '/'))
	strcpy(p_name, libdir);
      else	
	strcpy(p_name, "");

      strcat(p_name, name);
    }
  else
    {
      /* No, add the .so */
      p_name = (char *) calloc(len+4, sizeof(char));
      if ((libdir != NULL) && (name[0] != '/'))
	strcpy(p_name, libdir);
      else	
	strcpy(p_name, "");

      strcat(p_name, name);
      strcat(p_name, ".so");
      len+=3;
    }


  if (p_name == NULL) 
    return -1;
  
  p_name[len]='\0';



  handle = dlopen(p_name, RTLD_LAZY);
  if (! handle)
    {
      fprintf(stderr,"%s\n",dlerror());
      free(p_name);
      return -2;
    }
  if (p_name) free(p_name);


  dlerror();
  init_plugin = dlsym(handle,"init_plugin");
  if ((error=dlerror()) != NULL)
    {
      fprintf(stderr,"%s\n",error);
      return -2;
    }

  (*plugin_struct) = (*init_plugin)();
  
  if (*plugin_struct != NULL)
    (*plugin_struct)->handle = handle;
  else
    return -3;


  return 0;
}
