#ifndef PLUGIN_H
#define PLUGIN_H

#include <dlfcn.h>
#include <SDL.h>

/**
 * Plugin structure:
 ******************************************************************************
 *    Every plugin should contain a function init_plugin() that returns a 
 * (plugin *).
 *    The plugin type is defined above, but it basically consists of:
 *      - plot: this must point to the plugin plot function
 *      - info: information about the plugin and it's autor
 *      - config: information that must be passed to plot function
 *      - handle: this is to be used internally by the load_plugin() and 
 *           unload_plugin() functions, not the plugin init_plugin(). So the 
 *           plugin itself doens't have to use this pointer
 *
 */

struct information {
  char *name;               /* plugin's name */
  char *description;        /* plugin's description */
  char *version;            /* plugin's version */
  char *author;             /* author's name */
  char *url;                /* where to find the plugin source*/
};

typedef struct plugin_s {
  void (*plot)(void *config, SDL_Surface *, int, float *, int); /* plot (conf, surface, data_size, data[], rate) */
  struct information info;  /* information about the plugin */
  void *config;             /* config data */
  void *handle;             /* to be used internally */  
} plugin;


plugin *init_plugin();



/**
 * load_plugin: loads the plugin
 * @param name the name of the plugin or it's path 
 * @param plugin_struct where to return the plugin loaded 
 * @return 0 = ok, -1 = null pointer parameter, -2 = error loading plugin, -3 = error initializating plugin
 */
int load_plugin(char *name, plugin **plugin_struct);

/**
 * unload_plugin: unloads the plugin
 * @param plugin_struct pointer to the plugin loaded
 * @return 0 = ok, -1 = error
 */
int unload_plugin(plugin *plugin_struct);

#endif /* PLUGIN_H */
