#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <SDL.h>
#include "../plugin.h"

/* Frequency range to show */
#define FREQ_MIN    40.0f
#define FREQ_MAX 15000.0f
#define N_MAX    2048       /* maximum number of elements passed to this function */ 
#define MAX 30.0f
#define MIN  0.0f
#define LEVELS 10           /* maximum number of levels */
#define DELAY_USEC 50000    /* default delay */
#define N_BARS 20           /* default number of bars */
#define BAR_SPACING 1       /* default bar spacing */

struct basic_plot_cfg 
{
  long usec_delay;
  int levels;
  int n_bars;
  int bar_spacing;
  int color;
};


static void plot(void *config, SDL_Surface *screen, int data_size, float *data, int rate);
plugin *init_plugin();


static void plot(void *config, SDL_Surface *screen, int data_size, float *data, int rate)
{
  static short first_run = 1;
  static int bar_size=0, i=0, j=0, l=0, tmp_i=0, level=0;
  static float max_freq=0.0f;
  static float scale_y=0.0f, scale_x=0.0f, value=0.0f, value_old=0.0f;
  static SDL_Rect rectangle;
  static Uint32 color[LEVELS];
  static float data_levels[LEVELS][N_MAX];
  static double mult = 0.0;
  static struct basic_plot_cfg *cfg = NULL;
  
  if ( (screen == NULL) || (data == NULL))
    return;
  
  if (first_run)
    {
      cfg = (struct basic_plot_cfg *)config;

      color[0] = SDL_MapRGB(screen->format, cfg->color >> 16, cfg->color % 65536 / 256, cfg->color % 256);
      for (i=1; i < cfg->levels; i++)
	color[i] = SDL_MapRGB(screen->format, 
			      (100 - (int) 100*i/cfg->levels), 
			      (100 - (int) 100*i/cfg->levels), 
			      (100 - (int) 100*i/cfg->levels));
      /* this is black, used to erase the background */
      color[i] = SDL_MapRGB(screen->format, 0,0,0);
      if (data_size > N_MAX) 
	data_size = N_MAX;
      if (cfg->levels < LEVELS)
	cfg->levels = LEVELS;
    }

  
  if (mult == 0.0)
    mult = pow(10, log10(((double)FREQ_MAX/(double)FREQ_MIN))/(double)cfg->n_bars);
  

  for (i=0; i < data_size; i++) 
    { 
      data_levels[level][i] = data[i];			  
    }
      

  bar_size = (int) ((float)screen->w / cfg->n_bars) - cfg->bar_spacing; 
  scale_y = (float)screen->h / (MAX - MIN);
  scale_x = (float)screen->w / (cfg->n_bars * bar_size);

  SDL_LockSurface(screen);
  for (i=(cfg->levels-1); i >= 0; i--)
    {
      if (i == (cfg->levels-1))
	{
	  /* clear the screen */
	  rectangle.x = (Sint16) 0;
	  rectangle.y = (Sint16) 0;
	  rectangle.w = (Uint16) screen->w;
	  rectangle.h = (Uint16) screen->h; 
	  SDL_FillRect(screen, &rectangle, color[cfg->levels]);
	}
      else
	{
	  max_freq = FREQ_MIN;
	  value=0.0f;
	  l=0;
#ifdef DEBUG
	  printf("\n\n");
#endif
	  for (j=0; j < data_size; j++)
	    {
	      tmp_i = (int) ((float)rate*j/(2*data_size));
 	      if ( (tmp_i < max_freq) && (value < 10*log10(data_levels[(level+cfg->levels-i)%cfg->levels][j])) )
		{
		  value = 10*log10(data_levels[(level+cfg->levels-i)%cfg->levels][j]);
		  value_old = value;
		}
	      else if (tmp_i >= max_freq)
		{
		  /* if we have no value to this frequency, use the last one (some cases in low frequencies) */
		  if (value == 0.0f)
		    value = value_old; 

		  rectangle.x = (Sint16) l * (bar_size+ cfg->bar_spacing);
		  rectangle.y = (Sint16) screen->w - ceil((value-MIN)*scale_y);
		  rectangle.w = (Uint16) bar_size;
		  rectangle.h = (Uint16) ((float)(value-MIN))*scale_y;
		  SDL_FillRect(screen, &rectangle, color[i]);
#ifdef DEBUG		  
		  printf("    % -7.2fHz => % -5.3fdb (% -5.3f) (rate=% -5dHz; j=% -4d, N=% 4d)\n",
			 max_freq, value, (value-MIN), rate, j, data_size);
#endif
		  tmp_i ++;
		  max_freq *= mult;
		  j--;
		  l++;
		  value = 0.0f;
	      }
	    }
	}
    }
  SDL_UnlockSurface(screen); 
  SDL_UpdateRect(screen, 0, 0, screen->w, screen->h); 
  
  level = ++level % cfg->levels;

  first_run = 0;
}


plugin *init_plugin()
{
  plugin *plugin_struct = (plugin *) calloc(1, sizeof(plugin));
  static struct basic_plot_cfg config;
  

  plugin_struct->plot = plot;
  plugin_struct->handle = NULL;
  plugin_struct->info.name = "basic_plot";
  plugin_struct->info.description = "A very basic plot method: bar plot";
  plugin_struct->info.version = "1.0";
  plugin_struct->info.author = "Gustavo Sverzut Barbieri";
  plugin_struct->info.url = "<gsbarbieri@yahoo.com.br>";
 
  config.usec_delay = DELAY_USEC;
  config.levels = LEVELS;
  config.n_bars = N_BARS;
  config.bar_spacing = BAR_SPACING;
  config.color = (100 << 16) + (255 << 8) + 255;

  plugin_struct->config = &config;
  

  return plugin_struct;
}
